==> citellus-1.0.4.347462680.20200611091842.tar.gz: citellus-1.0.4.347462680.20200611091842/doc/development/CONTRIBUTING.md <==
**Table of contents**

<!-- TOC depthFrom:1 insertAnchor:false orderedList:false -->

- [How to file a bug report](#how-to-file-a-bug-report)
- [How to contribute code](#how-to-contribute-code)
- [How to write tests](#how-to-write-tests)
- [How to debug your test](#how-to-debug-your-test)

<!-- /TOC -->

## How to file a bug report

If you have any suggestions for improvements please do not hesitate to
open an [issue](https://github.com/citellusorg/citellus/issues/new).

## How to contribute code

We encourage you to contribute new plugins. We use [gerrithub][] for
reviewing proposed changes. The submission process looking something
like this:

[gerrithub]: https://gerrithub.io/

1.  Clone the citellus repository:
    `git clone https://github.com/citellusorg/citellus`
2.  Configure the `git-review` tool:
    `git-review -s`
3.  Install pre-commit (from pipsi for example) and prepare the hook:
    `pre-commit install`
4.  Check out a branch in which to make your changes:
    `git checkout -b "your-new-branch"`
5.  Edit your files and validate with tox:
    `tox # this will check the changes for some errors`
    NOTE: tox will run python 2.7, pep8 and python 3.5 tests, if your environment lacks for example python 3.5, do execute tox -l to see the available tests and skip that one, for example:

    ```sh
    tox -e pep8
    tox -e py27  # We're skipping tox -e py35 which is also invoked by default when tox is executed without arguments.
    ```

    In this way, errors that might be relevant to the test environment can be skipped. For example this one reported at issue tracker as [104](https://github.com/citellusorg/citellus/issues/104))

6.  Update your local repository:
    `git add $modified_files git commit`

        For the message, please use a short line with the fix and the subject like `[plugins][openstack][nova] Check nova configuration XXX`

        If the commit fixes a github open issue, also use `Closes #$ISSUEID` so github automatically closes it once merged referencing the commit.

7.  Submit your changes for review:

        git-review

Then wait for your changes to be reviewed. It is common for reviewers
to request changes; when this happens:

1.  Edit your files and revalidate with tox:

        tox # this will check the new changes for some errors

2.  Update your existing commit. Do not create a new commit!

        git add $modified_files
        git commit --amend

3.  Resubmit the change:

        git-review

You can see pending and already merged actual changes at: <https://review.gerrithub.io/q/citellusorg>

4. Once the new plugin has been submitted you'll see some comments from 'Citellus Jenkins' which is running Unit tests against it (same ones that you run with `tox`)

5. If Jenkins gives 'Verified +1', next step is wait for one reviewer to give final ACK and merge the change.

## How to write tests

Please refer to the
[templates](https://github.com/citellusorg/citellus/tree/master/doc/templates)
folder for examples.

Specially remember about the headers:

```
# long_name: plug long name for webui
# description: plug description
# bugzilla: bz url
# priority: 0<>1000 for likelihood to break your environment if this test reports fail
# kb: url-to-kbase
```

That are used by Citellus to fill json metadata.

If you want to contribute also Unittests for your plugins, check [TESTING.md](TESTING.md)

For contributing translations check [i18n.md](i18n.md)

## How to debug your test

We've included a file named `env-for.debug.sh` in the root folder of citellus repo that allows to be sourced and later execute your script.

This environment file will define the standard variables Citellus does use like:

- RC_OKAY
- RC_FAILED
- RC_SKIPPED
- RC_INFO
- TEXTDOMAIN
- TEXTOMAINDIR
- CITELLUS_BASE
- CITELLUS_LIVE
- CITELLUS_ROOT

And will preload the common-functions

The way to use it is:

```sh
. ~/citellus/env-for-debug.sh
```

Then you can debug your script with:

```
sh -x /path/to/your/plugin.sh
```

Or test individual functions output like:

```
is_rpm qemu-kvm-rhev
```

Please, do note that as some functions do 'exit' or 'return' for example executing `is_rpm_over` will exit the active shell, so beware!!

==> citellus-1.0.6.370299308.20200810122527.tar.gz: citellus-1.0.6.370299308.20200810122527/doc/development/CONTRIBUTING.md <==
**Table of contents**

<!-- TOC depthFrom:1 insertAnchor:false orderedList:false -->

- [How to file a bug report](#how-to-file-a-bug-report)
- [How to contribute code](#how-to-contribute-code)
- [How to write tests](#how-to-write-tests)
- [How to debug your test](#how-to-debug-your-test)

<!-- /TOC -->

## How to file a bug report

If you have any suggestions for improvements please do not hesitate to
open an [issue](https://github.com/citellusorg/citellus/issues/new).

## How to contribute code

We encourage you to contribute new plugins. Use Pull Requests to provide your contributions to the repo.

[gerrithub]: https://gerrithub.io/

1.  Clone the citellus repository:
    `git clone https://github.com/citellusorg/citellus`
2.  Install pre-commit (from pipsi for example) and prepare the hook:
    `pre-commit install`
3.  Check out a branch in which to make your changes:
    `git checkout -b "your-new-branch"`
4.  Edit your files and validate with tox:
    `tox # this will check the changes for some errors`
    NOTE: tox will run python 2.7, pep8 and python 3.5 tests, if your environment lacks for example python 3.5, do execute tox -l to see the available tests and skip that one, for example:

    ```sh
    tox -e pep8
    tox -e py27  # We're skipping tox -e py35 which is also invoked by default when tox is executed without arguments.
    ```

    In this way, errors that might be relevant to the test environment can be skipped. For example this one reported at issue tracker as [104](https://github.com/citellusorg/citellus/issues/104))

5.  Update your local repository:
    `git add $modified_files git commit`

        For the message, please use a short line with the fix and the subject like `[plugins][openstack][nova] Check nova configuration XXX`

        If the commit fixes a github open issue, also use `Closes #$ISSUEID` so github automatically closes it once merged referencing the commit.

6.  Submit your changes for review:

    `git push`

7.  Then, use repository Pull-Request method to request the change to be incorporated/reviewed

        git-review

Then wait for your changes to be reviewed. It is common for reviewers
to request changes; when this happens:

1.  Edit your files and revalidate with tox:

        tox # this will check the new changes for some errors

2.  Update your existing commit. Do not create a new commit! We want to get each new PR into one commit.

        git add $modified_files
        git commit --amend

3.  Resubmit the change:

        git push --force

4) Once the new plugin has been submitted you'll see some GitHub actions feedback (similars to the ones that you run with `tox`)

## How to write tests

Please refer to the
[templates](https://github.com/citellusorg/citellus/tree/master/doc/templates)
folder for examples.

Specially remember about the headers:

```
# long_name: plug long name for webui
# description: plug description
# bugzilla: bz url
# priority: 0<>1000 for likelihood to break your environment if this test reports fail
# kb: url-to-kbase
```

That are used by Citellus to fill json metadata.

If you want to contribute also Unittests for your plugins, check [TESTING.md](TESTING.md)

For contributing translations check [i18n.md](i18n.md)

## How to debug your test

We've included a file named `env-for.debug.sh` in the root folder of Citellus repo that allows to be sourced and later execute your script.

This environment file will define the standard variables Citellus does use like:

- RC_OKAY
- RC_FAILED
- RC_SKIPPED
- RC_INFO
- TEXTDOMAIN
- TEXTOMAINDIR
- CITELLUS_BASE
- CITELLUS_LIVE
- CITELLUS_ROOT

And will preload the common-functions

The way to use it is:

```sh
. ~/citellus/env-for-debug.sh
```

Then you can debug your script with:

```
sh -x /path/to/your/plugin.sh
```

Or test individual functions output like:

```
is_rpm qemu-kvm-rhev
```

Please, do note that as some functions do 'exit' or 'return' for example executing `is_rpm_over` will exit the active shell, so beware!!
